@@ -1,10 +1,10 @@
 # sqlalchemy/log.py
-# Copyright (C) 2006-2021 the SQLAlchemy authors and contributors
+# Copyright (C) 2006-2022 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 # Includes alterations by Vinay Sajip vinay_sajip@yahoo.co.uk
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
 
 """Logging control and utilities.
 
@@ -21,6 +21,7 @@ instance only.
 import logging
 import sys
 
+from .util import py38
 
 # set initial level to WARN.  This so that
 # log statements don't occur in the absence of explicit
@@ -160,6 +161,9 @@ class InstanceLogger(object):
             selected_level = self.logger.getEffectiveLevel()
 
         if level >= selected_level:
+            if py38:
+                kwargs["stacklevel"] = kwargs.get("stacklevel", 1) + 1
+
             self.logger._log(level, msg, args, **kwargs)
 
     def isEnabledFor(self, level):
