@@ -13,32 +13,18 @@ from sqlalchemy.orm import defer
 from sqlalchemy.orm import join as orm_join
 from sqlalchemy.orm import joinedload
 from sqlalchemy.orm import Load
-from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import selectinload
 from sqlalchemy.orm import Session
 from sqlalchemy.orm import sessionmaker
-from sqlalchemy.testing import config
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import profiling
 from sqlalchemy.testing.fixtures import fixture_session
+from sqlalchemy.testing.fixtures import NoCache
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 
 
-class NoCache(object):
-    run_setup_bind = "each"
-
-    @classmethod
-    def setup_test_class(cls):
-        cls._cache = config.db._compiled_cache
-        config.db._compiled_cache = None
-
-    @classmethod
-    def teardown_test_class(cls):
-        config.db._compiled_cache = cls._cache
-
-
 class MergeTest(NoCache, fixtures.MappedTest):
     __requires__ = ("python_profiling_backend",)
 
@@ -81,12 +67,12 @@ class MergeTest(NoCache, fixtures.MappedTest):
             cls.tables.child,
         )
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Parent,
             parent,
             properties={"children": relationship(Child, backref="parent")},
         )
-        mapper(Child, child)
+        cls.mapper_registry.map_imperatively(Child, child)
 
     @classmethod
     def insert_data(cls, connection):
@@ -102,7 +88,7 @@ class MergeTest(NoCache, fixtures.MappedTest):
 
         sess = fixture_session()
         sess2 = fixture_session()
-        p1 = sess.query(Parent).get(1)
+        p1 = sess.get(Parent, 1)
         p1.children
 
         # down from 185 on this this is a small slice of a usually
@@ -131,7 +117,7 @@ class MergeTest(NoCache, fixtures.MappedTest):
 
         sess = fixture_session()
         sess2 = fixture_session()
-        p1 = sess.query(Parent).get(1)
+        p1 = sess.get(Parent, 1)
         p1.children
 
         # preloading of collection took this down from 1728 to 1192
@@ -155,7 +141,7 @@ class MergeTest(NoCache, fixtures.MappedTest):
         self.assert_sql_count(testing.db, go2, 2)
 
 
-class LoadManyToOneFromIdentityTest(NoCache, fixtures.MappedTest):
+class LoadManyToOneFromIdentityTest(fixtures.MappedTest):
 
     """test overhead associated with many-to-one fetches.
 
@@ -202,8 +188,10 @@ class LoadManyToOneFromIdentityTest(NoCache, fixtures.MappedTest):
             cls.tables.child,
         )
 
-        mapper(Parent, parent, properties={"child": relationship(Child)})
-        mapper(Child, child)
+        cls.mapper_registry.map_imperatively(
+            Parent, parent, properties={"child": relationship(Child)}
+        )
+        cls.mapper_registry.map_imperatively(Child, child)
 
     @classmethod
     def insert_data(cls, connection):
@@ -297,7 +285,7 @@ class MergeBackrefsTest(NoCache, fixtures.MappedTest):
     def setup_mappers(cls):
         A, B, C, D = cls.classes.A, cls.classes.B, cls.classes.C, cls.classes.D
         a, b, c, d = cls.tables.a, cls.tables.b, cls.tables.c, cls.tables.d
-        mapper(
+        cls.mapper_registry.map_imperatively(
             A,
             a,
             properties={
@@ -306,9 +294,9 @@ class MergeBackrefsTest(NoCache, fixtures.MappedTest):
                 "ds": relationship(D, backref="a"),
             },
         )
-        mapper(B, b)
-        mapper(C, c)
-        mapper(D, d)
+        cls.mapper_registry.map_imperatively(B, b)
+        cls.mapper_registry.map_imperatively(C, c)
+        cls.mapper_registry.map_imperatively(D, d)
 
     @classmethod
     def insert_data(cls, connection):
@@ -374,7 +362,7 @@ class DeferOptionsTest(NoCache, fixtures.MappedTest):
     def setup_mappers(cls):
         A = cls.classes.A
         a = cls.tables.a
-        mapper(A, a)
+        cls.mapper_registry.map_imperatively(A, a)
 
     @classmethod
     def insert_data(cls, connection):
@@ -454,12 +442,12 @@ class AttributeOverheadTest(NoCache, fixtures.MappedTest):
             cls.tables.child,
         )
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Parent,
             parent,
             properties={"children": relationship(Child, backref="parent")},
         )
-        mapper(Child, child)
+        cls.mapper_registry.map_imperatively(Child, child)
 
     def test_attribute_set(self):
         Parent, Child = self.classes.Parent, self.classes.Child
@@ -533,12 +521,12 @@ class SessionTest(NoCache, fixtures.MappedTest):
             cls.tables.child,
         )
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Parent,
             parent,
             properties={"children": relationship(Child, backref="parent")},
         )
-        mapper(Child, child)
+        cls.mapper_registry.map_imperatively(Child, child)
 
     def test_expire_lots(self):
         Parent, Child = self.classes.Parent, self.classes.Child
@@ -584,7 +572,7 @@ class QueryTest(NoCache, fixtures.MappedTest):
         Parent = cls.classes.Parent
         parent = cls.tables.parent
 
-        mapper(Parent, parent)
+        cls.mapper_registry.map_imperatively(Parent, parent)
 
     def _fixture(self):
         Parent = self.classes.Parent
@@ -681,9 +669,13 @@ class SelectInEagerLoadTest(NoCache, fixtures.MappedTest):
         A, B, C = cls.classes("A", "B", "C")
         a, b, c = cls.tables("a", "b", "c")
 
-        mapper(A, a, properties={"bs": relationship(B)})
-        mapper(B, b, properties={"cs": relationship(C)})
-        mapper(C, c)
+        cls.mapper_registry.map_imperatively(
+            A, a, properties={"bs": relationship(B)}
+        )
+        cls.mapper_registry.map_imperatively(
+            B, b, properties={"cs": relationship(C)}
+        )
+        cls.mapper_registry.map_imperatively(C, c)
 
     @classmethod
     def insert_data(cls, connection):
@@ -812,13 +804,21 @@ class JoinedEagerLoadTest(NoCache, fixtures.MappedTest):
         A, B, C, D, E, F, G = cls.classes("A", "B", "C", "D", "E", "F", "G")
         a, b, c, d, e, f, g = cls.tables("a", "b", "c", "d", "e", "f", "g")
 
-        mapper(A, a, properties={"bs": relationship(B), "es": relationship(E)})
-        mapper(B, b, properties={"cs": relationship(C)})
-        mapper(C, c, properties={"ds": relationship(D)})
-        mapper(D, d)
-        mapper(E, e, properties={"fs": relationship(F), "gs": relationship(G)})
-        mapper(F, f)
-        mapper(G, g)
+        cls.mapper_registry.map_imperatively(
+            A, a, properties={"bs": relationship(B), "es": relationship(E)}
+        )
+        cls.mapper_registry.map_imperatively(
+            B, b, properties={"cs": relationship(C)}
+        )
+        cls.mapper_registry.map_imperatively(
+            C, c, properties={"ds": relationship(D)}
+        )
+        cls.mapper_registry.map_imperatively(D, d)
+        cls.mapper_registry.map_imperatively(
+            E, e, properties={"fs": relationship(F), "gs": relationship(G)}
+        )
+        cls.mapper_registry.map_imperatively(F, f)
+        cls.mapper_registry.map_imperatively(G, g)
 
     @classmethod
     def insert_data(cls, connection):
@@ -1107,8 +1107,10 @@ class BranchedOptionTest(NoCache, fixtures.MappedTest):
         A, B, C, D, E, F, G = cls.classes("A", "B", "C", "D", "E", "F", "G")
         a, b, c, d, e, f, g = cls.tables("a", "b", "c", "d", "e", "f", "g")
 
-        mapper(A, a, properties={"bs": relationship(B), "gs": relationship(G)})
-        mapper(
+        cls.mapper_registry.map_imperatively(
+            A, a, properties={"bs": relationship(B), "gs": relationship(G)}
+        )
+        cls.mapper_registry.map_imperatively(
             B,
             b,
             properties={
@@ -1118,11 +1120,11 @@ class BranchedOptionTest(NoCache, fixtures.MappedTest):
                 "fs": relationship(F),
             },
         )
-        mapper(C, c)
-        mapper(D, d)
-        mapper(E, e)
-        mapper(F, f)
-        mapper(G, g)
+        cls.mapper_registry.map_imperatively(C, c)
+        cls.mapper_registry.map_imperatively(D, d)
+        cls.mapper_registry.map_imperatively(E, e)
+        cls.mapper_registry.map_imperatively(F, f)
+        cls.mapper_registry.map_imperatively(G, g)
 
         configure_mappers()
 
@@ -1205,7 +1207,7 @@ class AnnotatedOverheadTest(NoCache, fixtures.MappedTest):
         A = cls.classes.A
         a = cls.tables.a
 
-        mapper(A, a)
+        cls.mapper_registry.map_imperatively(A, a)
 
     @classmethod
     def insert_data(cls, connection):
@@ -1223,7 +1225,9 @@ class AnnotatedOverheadTest(NoCache, fixtures.MappedTest):
         @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
-                q.all()
+                # test counts assume objects remain in the session
+                # from previous run
+                r = q.all()  # noqa F841
 
         go()
 
@@ -1237,7 +1241,9 @@ class AnnotatedOverheadTest(NoCache, fixtures.MappedTest):
         @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
-                q.all()
+                # test counts assume objects remain in the session
+                # from previous run
+                r = q.all()  # noqa F841
 
         go()
 
@@ -1249,7 +1255,9 @@ class AnnotatedOverheadTest(NoCache, fixtures.MappedTest):
         @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
-                q.all()
+                # test counts assume objects remain in the session
+                # from previous run
+                r = q.all()  # noqa F841
 
         go()
 
@@ -1261,7 +1269,9 @@ class AnnotatedOverheadTest(NoCache, fixtures.MappedTest):
         @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
-                q.all()
+                # test counts assume objects remain in the session
+                # from previous run
+                r = q.all()  # noqa F841
 
         go()
 
@@ -1274,7 +1284,9 @@ class AnnotatedOverheadTest(NoCache, fixtures.MappedTest):
         @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
-                q.all()
+                # test counts assume objects remain in the session
+                # from previous run
+                r = q.all()  # noqa F841
 
         go()
 
@@ -1287,7 +1299,9 @@ class AnnotatedOverheadTest(NoCache, fixtures.MappedTest):
         @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
-                q.all()
+                # test counts assume objects remain in the session
+                # from previous run
+                r = q.all()  # noqa F841
 
         go()
 
@@ -1299,7 +1313,9 @@ class AnnotatedOverheadTest(NoCache, fixtures.MappedTest):
         @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
-                q.all()
+                # test counts assume objects remain in the session
+                # from previous run
+                r = q.all()  # noqa F841
 
         go()
 
@@ -1312,7 +1328,9 @@ class AnnotatedOverheadTest(NoCache, fixtures.MappedTest):
         @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
-                q.all()
+                # test counts assume objects remain in the session
+                # from previous run
+                r = q.all()  # noqa F841
 
         go()
 
@@ -1324,6 +1342,8 @@ class AnnotatedOverheadTest(NoCache, fixtures.MappedTest):
         @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
-                q.all()
+                # test counts assume objects remain in the session
+                # from previous run
+                r = q.all()  # noqa F841
 
         go()
